/**
 * Creates an activator from a group of widgets
 * The Longman activator is made of one or more indexes, a search box, a result
 * list, and a "Concept" and "Section" panes
 * For instance:
 *
      <!-- search box -->
      <textbox  id="textboxSearch" group="activatorSearch" autoscroll="true"/>
      <!-- triggers the search -->
      <button group="activatorSearch"/>

      <!-- alpha index -->
      <skVirtualList id="indexActivatorKey"
        src="chrome://ldoce5/content/iframe/frameActivatorIndex.xml" flex="1"
        group="activatorSearch" type="index"
        fs="data/activator.skn/alpha_index_key.skn"/>

      <!-- result list -->
      <skVirtualList id="searchResultActivatorKey"
        src="chrome://ldoce5/content/iframe/frameActivatorRs.xml" flex="1"
        group="activatorSearch" type="results"
        fs="data/activator.skn/arl.skn"/>

      <!-- concept list (main panel) -->
      <skVirtualList class="mainBox" id="conceptActivator"
        src="chrome://ldoce5/content/iframe/frameActivatorConcept.xml" flex="1"
        group="activatorSearch"
        type="concept"
        fs="data/activator.skn/activator_concept.skn"/>

      <!-- section list (main panel) -->
      <skVirtualList class="rightBox" id="sectionActivator"
        src="chrome://ldoce5/content/iframe/frameActivatorSection.xml" flex="1"
        group="activatorSearch"
        type="section"
        fs="data/activator.skn/activator_section.skn"/>
 
 *
 * Don't forget to add the CSS rule for the "Section" cross reference XBL in
 * the iframe of the "Concept" virtual list. As the "Concept" works as some
 * sort of menu for the "Sections", this lets the user open a particular
 * section from a concept by clicking on a "Section" tag of the concept.
 * See activator.xml for the code to insert.
 */
function ActivatorSearchSystem(id, opts) {
    // JS inheritance
    this.base = SearchSystem;
    this.base(id,opts);
    this.scrollMain = false;
    var me = this;
    // doesn't seem to serve much purpose...
    this.conceptIndex = 0;
    this.sectionIndex = 0;
    // main panel
    this.initConcept();
    // right panel
    this.initSection();
    // Overload the callback on the result list, because the search system
    // relies on having one list to display items in, while we have 2
    this.resultList.selectItemCallback(function(item, obj) {
        try {
            me.loadConceptSection(item.index);
        }
        catch(e) {
           jsLog("Error loading concept/section for the " +
               "activator " + me.id + ": " + e);
        }
    });
    // Add internal xref handler, so that clicking on an internal xref in the
    // concept panel will highlight the target and load the appropriate section
    if (window.skResourceManager) {
        skResourceManager.setSelfHandler(skResourceManager.resolvePath(this.fsConcept),
            function(element, resource, topic, bookmark) {
                skResourceManager.scrollHandler(element, resource, topic,
                    bookmark);
                // find the search system for the activator and highlight the
                // target
                var selection = element.selection;
                if (selection)
                    setTimeout(function() {
                            try {
                                // do that for want of a better system to handle
                                // several views for the same resources
                                // grrr
                                var me = skGroups.selectedGroup.searchSystem;
                                var targetElement = me.conceptList.skVlist.
                                    contentDocument.getElementById(selection);
                                if (targetElement)
                                    targetElement.setAttribute("active", true);
                                me.loadSection(selection);
                            }
                            catch(e) {
                                alert(e);
                            }
                        }, 500);
        });
    }
    // Same for the callback on the index. Also makes sure we don't display the
    // results of the search, but directly load the concept and section linked
    // to the first result
    /*this.indexList.selectItemCallback(function(item, obj){
        try{
            // do a search - the search system will have the wanted result as
            // the first element
            searchString = skTabManager.getFieldValue(me.fsIdx,
                item.index, "ARL");
            if (!searchString)
                throw "Can't find the element associated with this entry, no " +
                    "search string found";
            me.searchResults = me.search(searchString, null, null,
                    false);
            // load the concept and section associated with the first result
            if (me.searchResults && me.searchResults.length > 0)
               me.loadConceptSection(0); 
            else
                throw "Oups... we should have found at least one record "
                    "associated with this search string: " + searchString +
                    " - invalid data?";
        } catch (err) {jsLog(err) }
    });
    */
}
ActivatorSearchSystem.prototype = new SearchSystem;

/** From the ARL, retrieves the result at position <index> and loads the concept
 * and section it is associated with */
ActivatorSearchSystem.prototype.loadConceptSection = function(index) {
    var aIds = skTabManager.getFieldValuesArray(this.fsArl,
        this.searchResults[index], ["concept_id",
            "concept_scroll_id", "section_id", "section_scroll_id"]);
    // These ids are publisher ids
    var conceptPubId = aIds[0];
    var conceptScrollId = aIds[1];
    var sectionPubId = aIds[2];
    var sectionScrollId = aIds[3];
    this.loadConcept(conceptPubId, conceptScrollId);
    this.loadSection(sectionPubId, sectionScrollId);
}

/**
 * Extends SearchSystem.assignWidget to integrate activator-specific widgets
 */
ActivatorSearchSystem.prototype.assignWidget = function (widget) {
    try {
        // sacrifices a chicken by a moonless night to the Ancient Ones,
        // to simulate 'super'
        if (!SearchSystem.prototype.assignWidget.call(this, widget)) {
            // if it's not one of the basic searchSystem widgets, then it must
            // be a 'concept' (main panel) or 'section' (right panel) vlist
            var type = $(widget).attr("type");
            if (type == "concept")
                this.concept = widget;
            else if (type == "section")
                this.section = widget;
            else
                return false;
            widget.searchSystem = this;
            return true;
        }
        return true;
    }
    catch (e) {
        throw "Error in ActivatorSearchSystem.assignWidget, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}
/**
 * Initializes the concept (main panel) virtual list
 */
ActivatorSearchSystem.prototype.initConcept = function() {
    if (!this.concept) {
        jsLog("WARNING: creating activator " + this.id +
                " without concept");
        return;
    }
    try {
        var me = this;
        this.fsConcept = this.prefix($(this.concept).attr("fs"));
        this.conceptList = new VList(this.concept);
        this.conceptList.stickySelection(true);
        this.conceptList.selectFirstItem(false);
        // one-item vlist
        this.conceptList.lastIndex(0);
        // don't mess with the scrollbars, let Moz handle them
        this.conceptList.skVlist.collapScroll = "true";
        this.conceptList.getContentCallback(function (piIndex, obj) {
            try{
                var entry   = skTabManager.getFile(me.fsConcept + "/config.cff",
                    piIndex).GetData().GetWStringData();
                entry = entry.replace(/^\s*<\?xml.+\??>/, '');
                // the virtual list sets a 'scrollId' field when it needs to scroll somewhere
                // if it does, we want to highligt the destination
                var scrollId = me.conceptList.skVlist.scrollId;
                if (scrollId)
                   entry = entry.replace(new RegExp("(<[^>]+?id=\"" + scrollId + "\")"), "$1 active=\"true\"");
                return new String(entry);
            } catch (err) {
                jsLog("Error in Concept getContent (index " +
                    me.conceptIndex + "): " + err);
            }
        });
    }
    catch (e) {
        throw "Error in ActivatorSearchSystem.initConcept, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}
/**
 * Initalizes the section (right panel) virtual list
 */
ActivatorSearchSystem.prototype.initSection = function() {
    if (!this.section) {
        jsLog("WARNING: creating activator " + this.id +
                " without section");
        return;
    }
    try {
        var me = this;
        this.fsSection = this.prefix($(this.section).attr("fs"));
        this.sectionList = new VList(this.section);
        this.sectionList.stickySelection(true);
        this.sectionList.selectFirstItem(false);
        // one-item vlist
        this.sectionList.lastIndex(0);
        // don't mess with the scrollbars, let Moz handle them
        this.sectionList.skVlist.collapScroll = "true";
        this.sectionList.getContentCallback(function (piIndex, obj) {
            try{
                var entry   = skTabManager.getFile(me.fsSection + "/config.cff",
                    me.sectionIndex).GetData().GetWStringData();
                entry = entry.replace(/^\s*<\?xml.+\??>/, '');
                // the virtual list sets a 'scrollId' field when it needs to scroll somewhere
                // if it does, we want to highligt the destination
                var scrollId = me.sectionList.skVlist.scrollId;
                if (scrollId)
                   entry = entry.replace(new RegExp("(<[^>]+?id=\"" + scrollId + "\")"), "$1 active=\"true\"");
                return new String(entry);
            } catch (err) {
                jsLog("Error in Section getContent (index " +
                    me.sectionIndex + "): " + err);
            }
        });
    }
    catch (e) {
        throw "Error in ActivatorSearchSystem.initSection, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}

/**
 * Loads a concept with a given publisher id
 */
ActivatorSearchSystem.prototype.loadConcept = function(pubId, scrollId) {
    if (!this.concept)
        return;
    try {
        // retrieve the IDM IDs matching conceptPubId and sectionPubId
        var idmId = skTabManager.getIdMapping(this.fsConcept,
            pubId);
        this.conceptIndex = Number(idmId);
        
        // scroll to the correct part of the entry
        this.conceptList.placeInVlist(this.conceptIndex, scrollId);
    }
    catch (e) {
        jsLog("Error loading concept " + pubId + ": " + e);
    }
}

/**
 * Loads a section with a given publisher id
 */
ActivatorSearchSystem.prototype.loadSection = function(pubId, scrollId) {
    if (!this.section)
        return;
    try {
        jsLog("Loading " + pubId + " into " + this.section.getAttribute("id"));
        // retrieve the IDM IDs matching sectionPubId and sectionPubId
        var idmId = skTabManager.getIdMapping(this.fsSection, pubId);
        this.sectionIndex = Number(idmId);
        // scroll to the correct part of the entry
        this.sectionList.placeInVlist(this.sectionIndex, scrollId);
    }
    catch (e) {
        jsLog("Error loading section " + pubId + ": " + e);
    }
}

/**
 * Extends SearchSystem.copy to copy the section iframe
 */
ActivatorSearchSystem.prototype.copy = function() {
    try {
        if (skCopy) {
            if (!this.sectionList)
                return;
            var section = this.sectionList.skVlist.contentDocument.getElementsByTagName("Section")[0];
            skCopy.add(section);
            skCopy.putInClipoard("text/unicode");
        } else {
            throw "skCopy is undefined, please include clipboard.js to make working " +
                "the copy through the searchSystem.";
        }
    }
    catch(e) {
        throw "Error in ActivatorSearchSystem.copy, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
};

/**
 * Extends SearchSystem.preparePrint to print the section iframe
 */
ActivatorSearchSystem.prototype.preparePrint = function () {
    try {
        var html = this.sectionList.getCurrent();
        var domDoc = this.domParser.parseFromString("<div xmlns:p='Edi' xmlns='http://www.w3.org/1999/xhtml'>" + html + "</div>", "text/xml");
        return domDoc;
    }
    catch (e) {
        throw "Error in ActivatorSearchSystem.preparePrint, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}

