/** 
 * Widget grouping - in particular, useful to enable-disable several unrelated
 * widgets
 */
/************************************************************************
 * Groups widgets together. To include a widget in a group "foo", just put an
 * attribute group="foo" on it.
 ************************************************************************/
function WidgetGroup(id) {
    this.init(id);
}

// Real constructor
WidgetGroup.prototype.init = function(id) {
    this.id = id;
    this.widgets = [];
    this.widgetsFromWindow = {};
    this.refresh();
}

// Refreshes the list of elements
WidgetGroup.prototype.refresh = function() {
    this.refreshFromWin(window);
}

// Removes all elements belonging to a given window
WidgetGroup.prototype.removeFromWin = function(win) {
    if (!this.widgetsFromWindow[win])
        return;
    var widget, i;
    var winWidgets = this.widgetsFromWindow[win];
    // Clone the array of widgets belonging to win, because removeWidget is also
    // going to remove the widgets from this array one by one
    var widgetsToRemove = new Array(winWidgets.length);
    for (i in winWidgets)
        widgetsToRemove[i] = winWidgets[i];

    for (i in widgetsToRemove)
        this.removeWidget(widgetsToRemove[i]);
}

// Tries to remove a widget. Returns false if the widget doesn't belong to the
// group
WidgetGroup.prototype.removeWidget = function(widget) {
    var removedFromWidgets = false;
    // we maintain two lists, the list of all widgets and the list of widgets
    // per window - we need to remove the widget from both
    for (var i in this.widgets) {
        if (this.widgets[i] == widget) {
            this.widgets.splice(i, 1);
            removedFromWidgets = true;
            break;
        }
    }
    for (var win in this.widgetsFromWindow) {
        for (i in this.widgetsFromWindow[win]) {
            if (this.widgetsFromWindow[win][i] == widget) {
                // Oups, we never found this widget in the list of all widgets
                // Die a painful death
                if (!removedFromWidgets)
                    throw "The state of the widget group of id " + id +
                        " is inconsistent, a widget exists in widgetsFromWindow but not widgets";
                this.widgetsFromWindow[win].splice(i, 1);
                // We assume a widget can't be recorded for two windows...
                return true;
            }
        }
    }
    return false;
}

// Checks whether we have members of a group in a given window
WidgetGroup.prototype.refreshFromWin = function(win) {
    // find all elements with the "group" attribute
    if (!win.document.getElementsByAttribute) {
        var me = this;
        return;
    }
    var elements = win.document.getElementsByAttribute("group", "*");
    var groups;
    var splitGroups;
    // remove all elements belonging to this window first
    this.removeFromWin(win);
    this.widgetsFromWindow[win] = []; 
    for (var i = 0; i < elements.length ; i++) {
        groups = $(elements[i]).attr("group");
        // an element may belong to different groups
        splitGroups = groups.split(",");
        for (var j = 0 ; j < splitGroups.length ; j++) { 
            if (splitGroups[j] == this.id) {
                this.widgets.push(elements[i]);
                this.widgetsFromWindow[win].push(elements[i]);
                break;
            }
        }
    }
    return this.widgetsFromWindow[win];
}


// Add a handler for the widgets in this group
WidgetGroup.prototype.bind = function(eventType, handler) {
    for (var i = 0 ; i < this.widgets.length ; i++) {
        $(this.widgets[i]).bind(eventType, handler);
    }
}

// Trigger an event on the widgets in this group
WidgetGroup.prototype.trigger = function(eventType, src) {
    for (var i = 0 ; i < this.widgets.length ; i++)
        $(this.widgets[i]).trigger(eventType, src || this);
}

// Returns a widget by its id
WidgetGroup.prototype.widget = function(id) {
    for (var i = 0 ; i < this.widgets.length ; i++) {
        if (this.widgets[i].getAttribute("id") == id)
            return this.widgets[i];
    }
    return null;
}
/*********************************************************
 * A WidgetGroup which can be enabled or disabled
 * Per default, a disabled widget has one of the following applied to it:
 * - if containing the attribute 'ondisable="foo"', foo is eval-ed (use onenable
 *   for a custom action when enabling the item)
 * - if containing the attribute 'hideMe="true"', has display:none applied to it
 * - if containing the attribute 'disabledClass="foo"', is given the class 'foo'
 * - if inside a deck, get deselected
 * - is set to 'disabled' otherwise
 * 
 * It receives an id and a hash of options.
 * These options can be:
 * - createSearchSystem: a callback to create a search system associated with
 *   this group
 * - enableHook: triggered when the group is enabled
 * - disableHook: triggered when the group is disabled
 * - copyCallback: triggered when copy() is called on the group
 * - saveHistoryCallback: triggered by the application (for instance, from
 *   the searchSystem) 
 *********************************************************/
function ActiveGroup(id, opts) {
    // cover your eyes - JS inheritance in progress
    this.base = WidgetGroup;
    this.base(id);

    this.searchSystem = null;
    // create empty hash if opts is null to avoid checking for null everytime
    opts = opts || {};
    this.createSearchSystem = opts.createSearchSystem;
    this.enableHook = opts.enableHook || null;
    this.disableHook = opts.disableHook || null;
    this.copyCallback = opts.copyCallback || null;
    this.saveHistoryCallback = opts.saveHistoryCallback || null;
    // you can look now
    var me = this;
    // if a group is on a tab, add an event listener
    var tabGroups, widget, callback, tabsElement;
    for (var i = 0; i < this.widgets.length; i++) {
        widget = this.widgets[i];
        if (widget.nodeName != "tab")
            continue;
        tabGroups = $(widget).attr("group").split(",");
        // only activate the group if the current group is the first group it belongs to
        // eg, '<tab group="a,b">' will enable 'a' when clicked but not 'b'
        if (tabGroups[0] == this.id) {
            tabsElement = widget.parentNode;
            callback = function(e) {
                if (tabsElement.selectedItem != widget)
                    return;
                me.enable();
            };
            tabsElement.addEventListener("select", callback, true);
            if (tabsElement.selectedItem == widget) {
               setTimeout(function() {  me.enable() }, 200);
            }
        }
    }

    // what to do when enabling a group
    this.bind("enable_" + this.id, function() {
        //jsLog("Enabling " + this.nodeName + " within " + me.id);
        // try to be intelligent
        if ($(this).hasAttr("onenable"))
            eval($(this).attr("onenable"));
        else if ($(this).attr("hideMe") == "true")
            //$(this).removeAttr("style");
            this.collapsed = false;
        else if ($(this).attr("disabledClass"))
            $(this).removeClass($(this).attr("disabledClass"));
        else if ($(this).parent().nodeName() == "deck")
            $(this).parent().get().selectedPanel = $(this).get();
        else if (this.enable)
            this.enable(me);
        else $(this).removeAttr("disabled");
    });
 
    // what to do when disabling it
    this.bind("disable_" + this.id, function() {
        // try to be intelligent
        if ($(this).hasAttr("ondisable"))
            eval($(this).attr("ondisable"));
        else if ($(this).attr("hideMe") == "true" && !this.collapsed)
      //      $(this).attr("style", "display: none");
            this.collapsed = true;
        else if ($(this).hasAttr("disabledClass"))
            $(this).addClass($(this).attr("disabledClass"));
        else if ($(this).parent().nodeName() == "deck")
            $(this).parent().get().selectedPanel = null;
        else if (this.disable)
            this.disable(me);
        else $(this).attr("disabled", "true");
    });

    // helper methods
    this.enable = function(searchSystem) {
        if (this.createSearchSystem != null && this.searchSystem == null) {
            this.searchSystem = this.createSearchSystem();
            this.searchSystem.load();
        }
        this.trigger("enable_" + this.id, this);

        if (this.searchSystem && this.searchSystem.enableSearchSystem){
            this.searchSystem.enableSearchSystem(this);
        }
        if(this.searchSystem && searchSystem){
            this.searchSystem.duplicateSearchSystem(searchSystem);
        }
        //focus the textbox
        if(this.searchSystem && this.searchSystem.input)
            this.searchSystem.input.focus();
        
        if (this.enableHook)
            this.enableHook();
    }
    this.disable = function() {
        this.trigger("disable_" + this.id, this);
        if (this.searchSystem && this.searchSystem.disableSearchSystem)
            this.searchSystem.disableSearchSystem(this);
        if (this.disableHook)
            this.disableHook();
    }
    this.copy = function() {
        if (this.copyCallback)
           this.copyCallback();
       else if (this.searchSystem)
           this.searchSystem.copy();
       else
           throw("No way to copy group whose id=" + this.id);
    }
}
// more JS inheritance crap
ActiveGroup.prototype = new WidgetGroup;

ActiveGroup.prototype.refreshFromWin = function(win) {
    WidgetGroup.prototype.refreshFromWin.call(this, win);
}
/******************************************
 * A list of ActiveGroups.
 * Only one ActiveGroup can be enabled at a time
 **********************************************/
function ActiveGroupList(groupIds) {
    this.init(groupIds);
}
// Constructor
ActiveGroupList.prototype.init = function(groupIds) {
    this.groups = [];
    this.selectedGroup = null;
    if (groupIds) {
        for (var i in groupIds)
            this.add(groupIds[i]);
    }
} 
// Add a group (either an id of a group or a group object) to this list
ActiveGroupList.prototype.add = function(groupObject) {
    var id;
    var groupToAdd = null;
    if (groupObject instanceof String || typeof groupObject == "string")
        id = groupObject;
    else if (groupObject instanceof ActiveGroup) {
        id = groupObject.id;
        groupToAdd = groupObject;
    }
    else throw "Don't know what to do with this object: " + groupObject;
    // can't add an object with the same id
    for (var i in this.groups) {
        if (this.groups.id == id)
            return;
    }
    if (groupToAdd == null)
        groupToAdd = new ActiveGroup(id);
    this.groups.push(groupToAdd);
}

// Returns the group with the given id, or null
ActiveGroupList.prototype.group = function(id) {
    for (var i = 0 ; i < this.groups.length ; i++) {
        if (this.groups[i].id == id) {
            return this.groups[i];
        }
    }
    return null;
}

// Active the group with the given id
ActiveGroupList.prototype.enable = function(id, searchSystem) {
    var idx = -1;
    var groupToSelect = null;
    if (id != null) {
        groupToSelect = this.group(id);
        if (groupToSelect == -1)
            throw "No group with id " + id;
    }
    else
        groupToSelect = this.groups[0];
    if (groupToSelect == null)
        throw "No such group '" + id + "' in " + this.groups[0];
    // the first time, disable all other groups
    if (this.selectedGroup == null) {
        for (var i = 0 ; i < this.groups.length ; i++) {
            if (this.groups[i].id != id)
                this.groups[i].disable();
        }
    }
    else {
        // otherwise, disable the current group
        this.selectedGroup.disable();
    }
    this.selectedGroup = groupToSelect;
    // enable the new group
    this.selectedGroup.enable(searchSystem); 
}
var skGroups = new ActiveGroupList();

