/**
 * SearchSystem - groups together an ARL, an index(optional), an AZ virtual list
 * and a result list.
 * To create a SearchSystem, group the widgets together (with
 * group="<search_system_id>"). The index should have the attribute
 * type="index", the az list type="az", the result list type="resultList".
 * There's no need of a 'type' attribute for the inputbox or the button which
 * starts the search, or the combox/checkboxes for the advanced search.
 * 
 * Options: the option hash can use the following parameters:
 * - dataPrefix (default: null): for instance, ldoce5.data - discouraged, use
 *   skResourceManager instead
 * - filter (default: null): a filter function to exclude results (can be
 *   useful when you don't want everything in the simple search) - not used when
 *   the searchSystem has an enabled skAdvancedSearch module belonging to it 
 * - singleEntry (default: false): set it to true to display a single entry in
 *   the AZ list
 * - win (default: window): set it to the value of the window the search system
 *   belongs to (useful when you initialize a search system in a popup from the
 *   main window via an event listener - see the dblclick function for an
 *   example
 * - popupUrl (default: chrome://plugs/content/skDblClickPopup/xul/skDblClickPopup.xul):
 *   url for the double click popup 
 * - dblClickPopup (default: true): each skDblElement in the searchSystem will
 *   open in a popup (otherwise the searchSystem will scroll)
 * - gotoCallback (default: this.placeInAz): call back for the "go to entry"
 *   button in the double click popup
 * - searchFromIndex (default: true): do a search from the index? If set to
 *   false, just go to the element the index entry which has been clicked on is
 *   linked to
 * - spellLanguage (default: en): language of the spellchecker
 * - highlight: should we highlight the element we scroll to? (will use the 
 * 'highlight' field of the ARL per default, or the 'context' field if the
 * 'highlight' field is not present)
 * - popupNoResultMessage (default: No results found): message displayed when 
 *   the user double-clicks and no result is found
 * - noResultHook (default: null): method executed when no result is found
 * - autoPhrasal (default: true): if a phrasal search is included in the search
 *   system, try and fallback on the phrasal search when we don't find anything
 *   in the regular search
 * - maxResults: maximum number of results to examine (before the filter function is applied, before duplicates, etc) (default: null - no limit)
 */
function SearchSystem(id, opts) { this.init(id, opts); }

SearchSystem.prototype.init = function (id, opts) {
    // make an empty hash if no opts are given, avoids the necessity to check
    // for null everytime we want to affect a value
    opts = opts || {};
    // group id (all members of the search system should have an attribute
    // group="<id>"
    this.id = id;

    // Message displayed when the user double-clicks and no result is found
    this.popupNoResultMessage = opts.popupNoResultMessage || "No results found";

    // Method executed when no result is found
    this.noResultHook = opts.noResultHook || null;

    // Maximum number of results
    this.maxResults = opts.maxResults != null ? opts.maxResults : null;
    // fallback to the phrasal search if included
    this.autoPhrasal = opts.autoPhrasal != null ? opts.autoPhrasal : true;

    // optional data prefix, like ldoce5.data - discouraged, use skResourceManager instead
    this.dataPrefix = opts.dataPrefix || null;
    // url of the double click popup
    this.popupUrl = opts.popupUrl || "chrome://plugs/content/skDblClickPopup/xul/skDblClickPopup.xul";
    //url of the document used to print
    this.skPrintUrl = opts.skPrintUrl;
    // display a single entry in the main window?
    this.singleEntry = opts.singleEntry || false;
    // select the first entry
    this.selectFirstEntry = opts.selectFirstEntry || false;
    // select the first entry of the index
    this.selectFirstIndexEntry = true;
    // language of the spellchecker
    this.spellLanguage = opts.spellLanguage || "en";
    // highlight a matched element within the entry
    this.highlight = opts.highlight || false;
    // default scroll speed for the vlists
    this.defaultScrollSpeed = 10;
    // scroll in the main AZ from the index
    this.scrollMain = true;
    // callback for the gotoentry button of the double click popup
    this.gotoCallback = opts.gotoCallback || this.placeInAz;
    // do a search from the main index?
    this.searchFromIndex = opts.searchFromIndex != null ? opts.searchFromIndex : true;
    // filter function (to exclude elements from the search results) - not used
    // in the advanced search
    this.filter = opts.filter || null;
    //method called when the group of the searchsystem is enabled
    this.enable = opts.enable || null;
    this.dblClickPopup = opts.dblClickPopup != null ? opts.dblClickPopup : true;
    this.widgetGroup = new WidgetGroup(id);
    //
    this.note = opts.note || null;

    // initialized when we lookup the widgets of the group
    this.combos = [];
    this.checkboxes = [];
    // number of words minimum to try a phrasal search
    this.phrasalThreshold = 3;
    var type;
    var me = this;
    // fs for the result list
    this.fsResults = null;
    // if we have several search modes (eg: search only on cultural entries) 
    // without having controls (like checkboxes) for this, fill this array with the default advanced search codes
    this.defaultCodes = [];
    this.initWidgets();
/** TODO MultiSearchSystemAssociate
**
    this.linkWithOtherSearchSystem =[];
**
**/
};

/** TODO MultiSearchSystemAssociate
**

// Add link with other search system 
SearchSystem.prototype.addSearchSystem = function (searchSystem) {
    this.linkWithOtherSearchSystem.push(searchSystem);
}
// get link with other search system 
SearchSystem.prototype.getSearchSystemLink = function (searchSystem) {
    return linkSearchSystem;
}
**
**/


/** get link with other search system */
SearchSystem.prototype.duplicateSearchSystem = function (searchSystem) {
    if(!searchSystem)
        return;
    if (searchSystem.input){
        this.input.value = searchSystem.input.value;
        if(this.input.value != "")
            this.searchAndDisplay(this.input.value);
    }
    var me = this;
    setTimeout( function(){
        if (searchSystem.index && me.index){
            me.indexList.placeInVlist(searchSystem.indexList.selectedIndex());
        }
        if (searchSystem.results && me.results){
            me.resultList.placeInVlist(searchSystem.resultList.selectedIndex());
        }
        if (searchSystem.az && me.az) {
            jsLog("Going to " + searchSystem.azList.selectedIndex());
            me.placeInAz(searchSystem.azList.selectedIndex(), null, null, true)
            }
    },250);
}

/** Refreshes this search system (call that if you add widgets) */
SearchSystem.prototype.refreshFromWin = function (win) {
    this.widgetGroup.refreshFromWin(win);
    this.initWidgets();
}

SearchSystem.prototype.initWidgets = function () {
    // find out which widgets do what
    var widgets = this.widgetGroup.widgets;
    for (var i = 0 ; i < widgets.length ; i++) {
        if (!this.assignWidget(widgets[i]))
            jsLog("WARNING: for the searchSystem of id '" + id + 
                    "', don't know what to do with this widget: " +
                    $(widgets[i]).nodeName());
    }

    this.domParser = new DOMParser();
    // fs for the result list
    this.fsResults = null;
    // id of the first entry currently displayed in the index list
    this.indexScrollId = null;
    // associate actions to widgets
    this.initInput();
    this.initButton();
    this.initSkDblElements();
    this.initPhrasalSearch();
    this.initIndex();
    // init result vlist
    this.initResults();
    // init main vlist
    this.initAz();
    // init Quickfind
    this.initQuickfind();
}

/**
 * Callback triggered when the group this search system belongs to is enabled
 */
SearchSystem.prototype.enableSearchSystem = function (group) {
    this.startQuickFind();
};

/**
 * Callback triggered when the group this search system belongs to is disabled
 */
SearchSystem.prototype.disableSearchSystem = function (group) {
    this.stopQuickFind();
};

/* Loops through the widgets belonging to the searchSystem, assigns them to
 * fields */
SearchSystem.prototype.assignWidget = function (widget) {
    try {
        var type = $(widget).attr("type");
        var nodeName = $(widget).nodeName();
        if (type == "index")
            this.index = widget;
        else if (type == "az")
            this.az = widget;
        else if (type == "results")
            this.results = widget;
        else if (nodeName == "textbox")
            this.input = widget;
        else if (nodeName == "skQuickFind")
            this.quickFind = widget;
        else if (nodeName == "select")
            this.combos.push(widget);
        else if (nodeName == "checkbox")
            this.checkboxes.push(widget);
        else if (nodeName == "button")
            this.button = widget;
        else if (nodeName == "skSpellPopup")
            this.spellpopup = widget;
        else if (nodeName == "skAdvancedSearch")
           this.skAdvancedSearch = widget;
        else if (nodeName == "skDblElement") {
           if (!this.skDblElements)
               this.skDblElements = [];
           this.skDblElements.push(widget);
        }
        else if (nodeName == "skPhrasalSearch")
            this.skPhrasalSearch = widget;
        else
            return false;
        // TODO remove this... a widget can belong to more than one search
        // system
        widget.searchSystem = this;
        return true;
    }
    catch(e) {
        throw "Error in assignWidget, in searchSystem " + this.id + 
            ": " + e;
    }
};

/**
* Assigns a callback to the quickfind component, if any
*/
SearchSystem.prototype.initQuickfind = function() {
   if (!this.quickFind)
       return;
   try {
       this.quickFind.disconnect();
       var me = this;
       this.quickFind.addListenerFunc(function(searchString) {
        searchString = searchString.replace(/\r\n/gi,"");
        searchString = searchString.replace(/\r/gi,"");
        searchString = searchString.replace(/\n/gi,"");
        //var sPrev = searchString.replace(/^.*<PREV>([^<]+)?.*/gi,"$1");
        var sWord = searchString.replace(/^.*<WORD>([^<]+)?.*/gi,"$1");
        //var sNext = searchString.replace(/^.*<NEXT>([^<]+)?.*/gi,"$1");
        if (String(sWord).replace(" ", "").length==0)
            return;
    /*    if (!sWord || sWord == "")
            return;*/
        me.input.value = sWord;
        me.searchAndDisplay(sWord);
       });
   }
   catch(e) {
       // no quickfind
       this.quickFind = null;
       jsLog("Error while initializing QuickFind: " + e);
   }
} 

/** Starts quickfind */
SearchSystem.prototype.startQuickFind = function() {
    try {
        if (this.quickFind)
            this.quickFind.turnQF();
    }
    catch(e) {
        throw "Error in startQuickFind, in searchSystem " + this.id + 
            ": " + e;
    }
};

/** Stops quickfind */
SearchSystem.prototype.stopQuickFind = function() {
    try {
        if(this.quickFind){
            this.quickFind.turnQF();
    }
    }
    catch(e) {
        throw "Error in stopQuickFind, in searchSystem " + this.id + 
            ": " + e;
    }
};

// Make sure that 'enter' can trigger the search and the scroll of the index
SearchSystem.prototype.initInput = function() {
    try {
        var me = this;
        $(this.input).keyup(function(e) {
            if (e.keyCode == 13){
                me.searchAndDisplay();
                if (me.azList)
                    me.azList.focus();
            }else{
                if($(me.input).attr("autoscroll")=="true"){
                    me.scroll();
                }
            }
        });
    }
    catch(e) {
        throw "Error in initInput, in searchSystem " + this.id + 
            ":" + e;
    }
};
/** Makes the button trigger the search */
SearchSystem.prototype.initButton = function() {
    if (!this.button || $(this.button).hasAttr("onclick"))
        return;
    try {
        // avoid weird problems with 'this' in Javascript
        var me = this;
        $(this.button).click(function() {
            me.searchAndDisplay();
        });
    }
    catch(e) {
        throw "Error in initButton, in searchSystem " + this.id + 
            ": " + e;
    }
};

SearchSystem.prototype.initSkDblElements = function() {
    if (!this.skDblElements)
        return;
    try {
        var me = this;
        for (var i in this.skDblElements)
           this.skDblElements[i].onDblSelection = function(text, evt, lang) { me.dblclick(text, evt, lang) };
    }
    catch(e) {
        throw "Error in initSkDblElements, in searchSystem " + this.id + 
            ": " + e;
    }
};

SearchSystem.prototype.initPhrasalSearch = function () {
    if (this.skPhrasalSearch)
        this.fsPhrasal = this.prefix(this.skPhrasalSearch.fs);
}

/**
 * Triggered on a double click in the AZ list
 * Either creates a popup or scrolls in the list
 */
SearchSystem.prototype.dblclick = function(text, evt, lang) {
    if (this.dblClickPopup) {
        if (this.popupUrl != null)
            this.dblclickPopup(text, evt, lang);
        else
            throw "No popup url specified for the double click, in " +
                "SearchSystem " + this.id;
    }
    else
        this.dblclickScroll(text, evt, lang);
}

/**
 * Triggered on a double click in the AZ list, if dblClickPopup equals 'true'.
 * Creates a popup with a search system, displaying the results of the search on
 * a word which has been double-clicked.
 */
SearchSystem.prototype.dblclickPopup = function(text, evt, lang) {
    try {
        var me = this;
        var newWindow;
        var dataPrefix = this.dataPrefix ||(window.skResourceManager ?
                window.skResourceManager.dataPrefix : null);
        if (skPopup.isAlreadyOpened("SK_DBLCLICK")){
            newWindow = skPopup.isAlreadyOpened("SK_DBLCLICK");
            try {
                var groupId = "dblclickSearch";
                var doc = newWindow.document;
                var resultLists = doc.getElementsByAttribute("type", "results");
                var vlist;
                // initialize the fs of the result list of the popup
                if (resultLists.length > 0) {
                   vlist = resultLists[0];
                   $(vlist).attr("fs", me.unprefix(me.fsArl));
                }
                var azLists = doc.getElementsByAttribute("type", "az");
                // initialize the fs of the az list of the popup
                if (azLists.length > 0) {
                   vlist = azLists[0];
                   $(vlist).attr("fs", me.unprefix(me.fsAz));
                }
                newWindow.searchSystem = new SearchSystem(groupId, {
                    dataPrefix: dataPrefix,
                    singleEntry: true,
                    filter: me.filter,
                    dblClickPopup: false,
                    maxResults: me.maxResults,
                });
                newWindow.searchSystem.refreshFromWin(newWindow);
                newWindow.searchSystem.load();
                var aResultsId = newWindow.searchSystem.search(
                    me.removeWildcards(text));
                var elements = doc.getElementsByAttribute("class", "dblclickpopup-resultbox");
                var element;
                if(elements.length>0)
                    element = elements[0];
                if (aResultsId.length == 0) {
                    newWindow.close();
                    if (this.popupNoResultMessage)
                        alert(this.popupNoResultMessage);
                }else if(aResultsId.length == 1 && element){
                    element.setAttribute("collapsed", "true");
                }else{
                    if(element)
                        element.removeAttribute("collapsed");
                }
                newWindow.searchSystem.display(aResultsId);
                //XXX - TODO : hide the results box instead of the results list itself!
                //if (aResultsId.length == 1)
                //   newWindow.searchSystem.azList.hide();
 
                // make the goto entry button do the right thing...
                var gotoEntryButton = newWindow.document.getElementById("goto");
                if (gotoEntryButton) {
                    gotoEntryButton.addEventListener("command", function(e) {
                        var curEntry = newWindow.searchSystem.resultList.selectedIndex();
                        jsLog(curEntry);
                        var fieldsToRetrieve = ["id"];
                        var aPathGuid = skTabManager.getFieldValuesArray(me.fsResults,
                            aResultsId[curEntry], fieldsToRetrieve);
                        if (typeof aPathGuid[0] == "number")
                            curEntry = aPathGuid[0];
                        else
                            curEntry = (skTabManager.getSkTable(me.fsAz + "/config.cff").
                                       GetFileByPath(aPathGuid[0])).GetFSObjectId();

                        if (me.gotoCallback)
                            me.gotoCallback(curEntry);
                        else
                            me.placeInAz(curEntry);
                   }, false);
                }
            } catch (e) {
               jsLog("Error while loading the double-click popup: " + e);
            }
            return;
        }
        var newPopup = skPopup.initNewPopup("SK_DBLCLICK", "POPUP_TYPE_GENERIC");
        newPopup.width  = 350;
        newPopup.height = 550;
        newPopup.url = this.popupUrl;
        // Test for evt, because it might not be present (eg, dbl click from
        // Flash module in LDOCE5)
        if(evt && window != evt.view)
            newPopup.winOpener = evt.view;
        //newPopup.extraArgs = { searchResults: this.search(text) };
        newWindow = skPopup.openPopup();
        // get the dataprefix from ourselves or from the resource manager
        newWindow.addEventListener("load", function loadDblclickSearch() {
            try {
                var groupId = "dblclickSearch";
                var doc = newWindow.document;
                var resultLists = doc.getElementsByAttribute("type", "results");
                var vlist;
                // initialize the fs of the result list of the popup
                if (resultLists.length > 0) {
                   vlist = resultLists[0];
                   if (!$(vlist).hasAttr("fs"))
                       $(vlist).attr("fs", me.unprefix(me.fsArl));
                }
                var azLists = doc.getElementsByAttribute("type", "az");
                // initialize the fs of the az list of the popup
                if (azLists.length > 0) {
                   vlist = azLists[0];
                   if (!$(vlist).hasAttr("fs"))
                       $(vlist).attr("fs", me.unprefix(me.fsAz));
                }
                newWindow.searchSystem = new SearchSystem(groupId, {
                    dataPrefix: dataPrefix,
                    singleEntry: true,
                    filter: me.filter,
                    dblClickPopup: false,
                });
                newWindow.searchSystem.refreshFromWin(newWindow);
                newWindow.searchSystem.load();
                var aResultsId = newWindow.searchSystem.search(
                    me.removeWildcards(text));
                var element = newWindow.document.getElementById("resultbox");
                if (aResultsId.length == 0) {
                    newWindow.close();
                    alert("No result found");
                }else if(aResultsId.length == 1 && element){
                    element.setAttribute("collapsed", "true");
                }else{
                    if(element)
                        element.removeAttribute("collapsed");
                }
                newWindow.searchSystem.display(aResultsId);
                //XXX - TODO : hide the results box instead of the results list itself!
                //if (aResultsId.length == 1)
                //   newWindow.searchSystem.azList.hide();
 
                // make the goto entry button do the right thing...
                var gotoEntryButton = newWindow.document.getElementById("goto");
                if (gotoEntryButton) {
                    gotoEntryButton.addEventListener("command", function(e) {
                        var curEntry = newWindow.searchSystem.resultList.selectedIndex();
                        var fieldsToRetrieve = ["id"];
                        var aPathGuid = skTabManager.getFieldValuesArray(me.fsResults,
                            newWindow.searchSystem.searchResults[curEntry], fieldsToRetrieve);
                        if (typeof aPathGuid[0] == "number")
                            curEntry = aPathGuid[0];
                        else
                            curEntry = (skTabManager.getSkTable(me.fsAz + "/config.cff").
                                       GetFileByPath(aPathGuid[0])).GetFSObjectId();

                        if (me.gotoCallback)
                            me.gotoCallback(curEntry);
                        else
                            me.placeInAz(curEntry);
                   }, false);
                }
            } catch (e) {
               jsLog("Error while loading the double-click popup: " + e);
            }
        }, false);
    } catch (e) {
        throw "Error in dblclickPopup, in searchSystem " + this.id + 
            ": " + e;
    }
}

/**
 * Scrolls through the virtual list on a double click
 */
SearchSystem.prototype.dblclickScroll = function(text, evt, lang) {
    try {
        this.searchAndDisplay(this.removeWildcards(text));
    } catch (e) {
        throw "Error in dblclickScroll, in searchSystem " + this.id + 
            ": " + e;
    }
}

/**
 * Removes wildcards from the search string.
 * Useful for double click, searching from index.
 */
SearchSystem.prototype.removeWildcards = function (searchString) {
    return searchString.replace(/[*?]/g, "");
}

/**
 * Adds the data prefix to the path, if any
 */
SearchSystem.prototype.prefix = function(path) {
    if (this.dataPrefix)
        return this.dataPrefix + "/" + path;
    else if (window.skResourceManager)
        return window.skResourceManager.prefix(path);
    else
        return path;
}

/**
 * Adds the data prefix to the path, if any
 */
SearchSystem.prototype.unprefix = function(path) {
    if (this.dataPrefix)
        return path.replace(new RegExp("^" + this.dataPrefix + "/"), "");
    else if (window.skResourceManager)
        return window.skResourceManager.unprefix(path);
    else
        return path;
}


// Initialize the index
SearchSystem.prototype.initIndex = function() {
    if(!this.index)
        return;
    try {
        var me = this;
        this.fsIdx = this.prefix($(this.index).attr("fs"));
        this.indexList = new VList(this.index);

        this.indexList.scrollSpeed(this.defaultScrollSpeed);
        this.indexList.lastIndex(skTabManager.getSkTable(this.fsIdx).
                GetCount() - 1);
        if (this.selectFirstIndexEntry)
            this.indexList.selectFirstItem(true);
        this.indexList.getContentCallback(function (piIndex, obj) {
            try{
                var entry   = skTabManager.getFieldValue(me.fsIdx, 
                    piIndex, "LABEL");
                return String(entry);
            } catch (err) {
                jsLog("GetEntryContent failed: " + err);
            }
        });
        this.indexList.getTooltipCallback(function(piIndex, obj){
            try{
                var entry =  skTabManager.getFieldValue(me.fsIdx, piIndex,
                    "TOOLTIP");
                entry = entry.replace(/&amp;/g,"&");
                return new String(entry);
            } catch (err) {jsLog(err) }
        });
        this.indexList.selectItemCallback(function(item, obj){
            try{
                var searchString;
                // the index might have a special ARL form
                try {
                    searchString = skTabManager.getFieldValue(me.fsIdx,
                    item.index, "ARL");
                }
                catch(e) {
                    // otherwise, use the lookup form
                    searchString = skTabManager.getFieldValue(me.fsIdx,
                    item.index, "SRCH");
                }
                var guidToSelect = skTabManager.getFieldValue(me.fsIdx,
                    item.index, "ID");
                // scroll in the main az list if needed
                //if (!me.searchFromIndex && !me.singleEntry && me.scrollMain)
                if (!me.searchFromIndex && me.azList)
                    me.azList.placeInVlist(guidToSelect);
                if (me.searchFromIndex)
                    me.searchAndDisplay(me.removeWildcards(searchString), null,
                        guidToSelect,
                        false);
            } catch (err) {jsLog(err) }
        });
        if($(this.index).hasAttr("fsLC"))
            this.lcFs = this.prefix($(this.index).attr("fsLC"));
        else {
            if ($(this.index).hasAttr("fs"))
                this.lcFs = this.prefix($(this.index).attr("fs") +
                        "/lc_SRCH.skn");
            else
                throw "No 'fs' or 'fsLc' attribute on the index list";
        }
    }
    catch(e) {
        throw "Error in initIndex, in searchSystem " + this.id + 
            ": " + e;
    }
};
// Initialize the main virtual list
SearchSystem.prototype.initAz = function() {
    if(!this.az)
        return;
    try {
        var me = this;
        this.fsAz = this.prefix($(this.az).attr("fs"));
        this.azList = new VList(this.az);
        if (this.selectFirstEntry) {
            this.azList.stickySelection(true);
            this.azList.selectFirstItem(true);
        }
        this.azList.scrollSpeed(this.defaultScrollSpeed);

        var lastIndex = null;
        if (this.singleEntry) {
           lastIndex = 0;
           this.azList.singleEntry();
        }
        else
            lastIndex = skTabManager.getSkTable(this.fsAz + 
                    "/files.skn").GetCount() - 1;
        this.azList.lastIndex(lastIndex);

        this.azList.getContentCallback(function (piIndex, obj) {
            try{
                var entry   = skTabManager.getFile(me.fsAz + "/config.cff",
                    piIndex).GetData().GetWStringData();
                entry = entry.replace(/^\s*<\?xml.+\??>/, '');
                if(this.note)
                    entry = this.note(entry);
                return String(entry);
            } catch (err) {
                jsLog("GetEntryContent failed: " + err + " ON piIndex : " +
                    piIndex);
            }
        });
        this.azList.getTooltipCallback(function (piIndex, obj){
            try{
                var entry =  skTabManager.getFieldValue(me.fsAz + "/files.skn",
                    piIndex, "TEXTTITLE");
                entry = entry.replace(/&amp;/g,"&");
                return new String(entry);
            } catch (err) {jsLog(err) }
        });
    }
    catch(e) {
        throw "Error in initAz, in searchSystem " + this.id + 
            ": " + e;
    }
};
// Init the result list
SearchSystem.prototype.initResults = function() {
    if (!this.results)
        return;
    var me = this;
    try {
        this.fsArl = this.prefix($(this.results).attr("fs"));
        // We do that so we can change fsResults to fsPhrasal when we need to
        // display the phrasal search
        this.fsResults = this.fsArl;
        this.resultList = new VList(this.results);
        this.resultList.scrollSpeed(this.defaultScrollSpeed);
        this.resultList.getContentCallback(function (piIndex, obj) {
            try{
                var entry = skTabManager.getFieldValue(me.fsResults,
                    me.searchResults[piIndex],"display");
                entry = entry.replace(/^\s*<\?xml.+\??>/, '');
                //jsLog("Entry content : " + new String(entry));
                return new String(entry);
            } catch (err) {
                jsLog("GetEntryContent failed: " + err);
            }
        });
        this.resultList.getTooltipCallback(function(piIndex, obj){
            try{
                var entry = skTabManager.getFieldValue(me.fsResults,
                    me.searchResults[piIndex],"display");
                entry = entry.replace(new RegExp("<[^>]+>","gi"),"");
                return new String(entry);
            } catch (err) { jsLog(err) }
        });
        this.resultList.selectItemCallback(function(item, obj){
            try{
                var hasBookmark = skTabManager.hasField(me.fsResults, "context");
                var hasHighlight = skTabManager.hasField(me.fsResults, "highlight");
                var fieldsToRetrieve = ["id"];
                if (hasBookmark)
                    fieldsToRetrieve.push("context");
                var aPathGuid = skTabManager.getFieldValuesArray(me.fsResults,
                    me.searchResults[item.index], fieldsToRetrieve);
                var id;
                if (typeof aPathGuid[0] == "number")
                    id = aPathGuid[0];
                else
                    id = (skTabManager.getSkTable(me.fsAz + "/config.cff").
                        GetFileByPath(aPathGuid[0])).GetFSObjectId();

                var context = aPathGuid[1];
                // there's a bug when the vlist has just one element
                // freaking setTimeout
                if (me.highlight) {
                    var idToHighlight;
                    if (hasHighlight){
                        idToHighlight = skTabManager.getFieldValue(me.fsResults, me.searchResults[item.index], "highlight");
                    }else
                        idToHighlight = context;
                }
                if (me.singleEntry)
                    setTimeout(function () { me.azList.placeInVlist(id, context, idToHighlight, true)}, 100);
                else{
                    me.azList.placeInVlist(id, context, idToHighlight);
                }
            } catch (err) { jsLog(err) }
        });
    }
    catch(e) {
        throw "Error in initResults, in searchSystem " + this.id + 
            ": " + e;
    }
};

// Does a search in the ARL and returns an array of results.
// 'codes' is optional advanced search codes
// 'idToSelect' is a publisher id of the item which should be at the top of the
// result list (useful when coming from the index)
// 'doSpellSearch' is a boolean telling if we should spellcheck the results
SearchSystem.prototype.search = function(searchString, codes, idToSelect, doSpellSearch) {
    try {
        this.fsResults = this.fsArl;
        var resultFs, indexFs;
        if(this.results){
            if($(this.results).hasAttr("fs"))
                resultFs = this.prefix($(this.results).attr("fs"));
            else
                throw "No 'fs' attribute on the result list";
            if($(this.results).hasAttr("fsIndex"))
                indexFs = this.prefix($(this.results).attr("fsIndex"));
            else
                indexFs = resultFs + "/idx_index.skn/index.cfi"
        }
        // default callback - does a search and returns the results in an array
        var i;
        if (!searchString) searchString = this.input.value;
        if (doSpellSearch == null) doSpellSearch = true;
        // add the advanced search codes
        if (!codes) {
            codes = [];
            // do we have combos in this search system?
            if (this.combos) {
                for (i = 0 ; i < this.combos.length; i++) {
                    if (!combos[i].disabled && 
                            !$(combos[i]).hasClass("disabled"))
                        codes.push(combos[i].value);
                }
            }
            // or checkboxes?
            if (this.checkboxes) {
                for (i = 0 ; i < this.checkboxes.length; i++)
                    if (!this.checkboxes[i].disabled && !$(this.checkboxes[i]).
                            hasClass("disabled"))
                        codes.push(this.checkboxes[i].value);
            }
            // or a full-blown advanced search system?
            if (this.skAdvancedSearch)
               codes = codes.concat(this.skAdvancedSearch.getSearchCodes());
        }

        this.codes = codes || []; 
        // Don't use the filter function if we have advanced search codes
        if(this.defaultCodes)
            this.codes = this.codes.concat(this.defaultCodes);
        if (!g_Simplifier || (this.codes.length == 0 && !searchString))
            return;
        // turn the array of advanced search codes into a string and add it to
        // the query
        var searchCodesString = this.codes.length > 0 ? 
            this.codes.join(" ") + " " :
            "";
        var query = searchCodesString + searchString.
            replace(/[\.;!()\+,\[:\]<>^`\-\[\]{}~]/g," ");
        var ir = skTabManager.getSkTable(indexFs, "|(&(!(^|<(").
            SearchExpression(query, true, g_Simplifier, null);
        var dl = ir.GetDocumentList();
        var aResultsId = new Array();
        var iResults   = dl.GetCount();
        var antiDupli  = {};
        var aField, GUID, clss, idx, display, index, filePath, idmId, klass;
        var isIdToSelect;
        // if we didn't click on an item of the index, try to position the match
        // whose index equals the searchString in the first position IF the
        // search string contains diacritics, hyphens, etc
        var matchSearchString = idToSelect ?
            false :
            (searchString && searchString.replace(/[|*?\s]/, "").match(/\W/));
        var hasIndex = skTabManager.hasField(resultFs, "index");
        var cleanDisplayRegExp = new RegExp(/^<span[^>]+>/);
        var matchIdRegExp = new RegExp(/(\d+).xml$/);
        var lastDisplay;
        var fieldsToRetrieve;
        var start = new Date();
        var retrieveTime = 0;
        var duplicateTime = 0;
        var duplicate;
        var removedResults = 0;
        for (i = 0; i < iResults; i++) {
            idx = dl.GetElement(i);
            if (this.maxResults != null) {
                if (i >= this.maxResults)
                    break;
            }
            // give it to the filter function if any
            if (this.filter) { 
                retrieveTime += this.benchmark(function() {
                    klass = Number(skTabManager.getFieldValue(resultFs, idx, "class"));
                });
                if (!this.filter(filePath, display, klass)) {
                    removedResults++;
                    continue;
                }
            }
            retrieveTime += this.benchmark(function() {
            display = skTabManager.getFieldValue(resultFs, idx, "display");
            });
            // we need to check if there has been another similar entry,
            // regarding of its class - as items are surround with
            // <span class="arlXX">...</span> - with XX the class number
            // we strip the span to make the check
            
            display = display.replace(cleanDisplayRegExp, '');
            // check if the display is identical to last time, can save us the
            // lookup in the hash, which is potentially expensive
            if (display == lastDisplay)
                continue;
            lastDisplay = display;
            duplicateTime += this.benchmark(function() {
                duplicate = (antiDupli[display] != null);
            });
            if (!duplicate) {
                isIdToSelect = false;
                if (idToSelect) {
                    retrieveTime += this.benchmark(function() {
                        filePath = skTabManager.getFieldValue(resultFs, idx, "id");
                    });
                    // Only retrieve the number at the end
                    if (!filePath.match(matchIdRegExp))
                        throw "Major error in the data: can't find IDM id in the path " + filePath;
                    idmId = Number(RegExp.$1);
                    isIdToSelect = (idmId == idToSelect);
                    // If we've found what we wanted, don't try to match anymore
                    if (isIdToSelect)
                        idToSelect = null;
                }
                else if (matchSearchString && hasIndex) {
                    retrieveTime += this.benchmark(function() {
                        index = skTabManager.getFieldValue(resultFs, idx, "index");
                    });
                    // Remove tags
                    index = index.replace(/<[^>]+>/g, " ");
                    // Remove adv search codes
                    index = index.replace(/\|\d+\|/g, " ");
                    // Remove spaces at the beginning*/
                    //index = index.replace(/^\s*/, "");
                    // Remove spaces at the end
                    index = index.replace(/\s*$/, "");
                    // Turn multiple spaces into single spaces
                    index = index.replace(/\s\s+/g, " ");
                    isIdToSelect = (index == searchString);
                    // If we've found what we wanted, don't try to match anymore
                    if (isIdToSelect)
                        matchSearchString = false;
                }
                // we don't want to see twice the same string in the search result vlist,
                // so we exclude any duplicates
                antiDupli[display] = true;
                if (!isIdToSelect)
                    aResultsId.push(idx);
                else
                    // selected id go in the first position
                    aResultsId.unshift(idx);
            }
        //aResultsId[i] = (idx);
        }
        var end = new Date();
        var result = new Date();
        result.setTime(end.getTime() - start.getTime());
        var totalTime = result.getTime();
        var s ="Search expression: '" + query + "'\nExamined " + iResults +
            " results in " + totalTime + " ms (" + (totalTime / iResults) +
            " ms/result), including:\n" ;
        s += formatBench("SK", retrieveTime, totalTime);
        s += formatBench("Duplicate removal", duplicateTime, totalTime);
        s += "\n\nRemoved " + (removedResults) + " results (" +
            ((removedResults/iResults)*100) + " %) - remains: " + aResultsId.length;
        jsLog(s)
        // Try phrasal search if enabled and the number of words is >= to
        // phrasalThreshold
        if (aResultsId.length == 0 && this.skPhrasalSearch && this.autoPhrasal &&
                searchString.split(" ").length >= this.phrasalThreshold) {
            aResultsId = this.phrasalSearch(searchString);
        }
        // Still no results? try spellchecking
        var foundSpellMatches = false;
        if (aResultsId.length == 0 && this.spellpopup && doSpellSearch) 
            foundSpellMatches = this.spellSearch(searchString);
        // trigger the hook if we really, really didn't find anything
        if (!foundSpellMatches && aResultsId.length == 0 && this.noResultHook)
            this.noResultHook(searchString, codes, idToSelect);
        this.searchResults = aResultsId;
        return aResultsId;
    }
    catch(e) {
        throw "Error in search, in searchSystem " + this.id +
            ": " + e;
    }
};
// small benchmark function
SearchSystem.prototype.benchmark = function(f) {
    var startDate = new Date();
    f();
    var endDate = new Date();
    return endDate.getTime() - startDate.getTime();
}

function formatBench(desc, time, totalTime) {
    return "- " + desc + ": " + time + " ms (" + ((time/totalTime) * 100) + " %)\n";
}
// Does a phrasal search in the phrasal search component
SearchSystem.prototype.phrasalSearch = function(search) {
    if (!this.skPhrasalSearch)
        throw "No skPhrasalSearch in the search system of id " + this.id;
    if (!search)
        search = this.input.value;
    // this way the results will be taken from the phrasal search fs
    this.fsResults = this.fsPhrasal;
    return this.skPhrasalSearch.search(search);
}

// Fires up the spell checker popup
SearchSystem.prototype.spellSearch = function(search) {
    try {
        var me = this;
        if (!search) search = this.input.value;
        if(search.length > 0 && this.spellpopup) {
            var dataPath = this.fsArl + "/spell/" + this.spellLanguage;

            if (this.spellpopup.dataDico != dataPath) {
                this.spellpopup.textbox  = this.input;
                this.spellpopup.dataDico = dataPath;
                this.spellpopup.init();
                this.spellpopup._showSelectedWord = function(win, poItem) {
                    me.searchAndDisplay(poItem.getAttribute("label"));
                    win.close();
                }
            }

            return this.spellpopup.checkWord(search, "mainSpell");
        }
        return false;
    }
    catch(e) {
        throw "Error in spellSearch, in searchSystem " + this.id +
            ": " + e;
    }
};
// Does a search and displays the results in the result list
SearchSystem.prototype.searchAndDisplay = function(searchString, codes, idToSelect, doSpellSearch) {
    var aResultsId = this.search(searchString, codes, idToSelect, doSpellSearch);
    this.display(aResultsId);
}

// Does a phrasal search and displays the results in the result list
SearchSystem.prototype.searchPhrasalAndDisplay = function(searchString) {
    var aResultsId = this.phrasalSearch(searchString);
    this.display(aResultsId);
}

// Assigns or returns the search results, they are also displayed in the
// result list
SearchSystem.prototype.display = function(searchResults) {
    try {
        this.resultList.clearList();
        if (searchResults == null)
            return this.searchResults;
        this.searchResults = searchResults;
        this.resultList.lastIndex(this.searchResults.length-1);
        this.resultList.load();
        // need to clear 'select' in the az list if it's a single entry
        // (otherwise the select event is not triggered)
        if (this.singleEntry && this.azList)
            this.azList.skVlist.selected = null;
    }
    catch(e) {
        throw "Error in display, in searchSystem " + this.id +
            ": " + e;
    }
};

// Scroll to the value in the input box
SearchSystem.prototype.scroll = function() {
    try {
        if (!this.index)
            return;
        if (!this.input || this.input.value=="") {
            this.indexList.selectedIndex(-1);
            return;
        }

        var arrayApproxId = getApproxIdSearch(this.input.value, this.lcFs, this.fsIdx, true);
        if (arrayApproxId.length > 1) {
            this.indexScrollId = arrayApproxId[1];
            this.indexList.placeInVlist(this.indexScrollId);
        }
    }
    catch(e) {
        throw "Error in scroll, in searchSystem " + this.id +
            ": " + e;
    }
};

// Scroll to the value in the input box
SearchSystem.prototype.placeInAz = function(id, context, highlight, force) {
    try {
        if (!this.azList)
            return;
        this.azList.placeInVlist(id, context, highlight, force);
    }
    catch(e) {
        throw "Error in placeInAz, in searchSystem " + this.id +
            ": " + e;
    }
};

// Populate the virtual lists
SearchSystem.prototype.load = function() {
    try {
        if (this.azList) this.azList.load();
        if (this.indexList) this.indexList.load();
    }
    catch(e) {
        throw "Error in load, in searchSystem " + this.id +
            ": " + e;
    }
};

// Copy the selected entry
SearchSystem.prototype.copy = function() {
    try {
        if (skCopy) {
            if (!this.azList)
                return;
            skCopy.add(this.azList.selectedItem());
            skCopy.putInClipoard("text/unicode");
        } else {
            throw "skCopy is undefined, please include clipboard.js to make working " +
                "the copy through the searchSystem.";
        }
    }
    catch(e) {
        throw "Error in copy, in searchSystem " + this.id + ": " + e;
    }
};

// Print the selected entry
SearchSystem.prototype.preparePrint = function() {
    try {
        if (this.azList) {
            var html = this.azList.getCurrent();
            var domDoc = this.domParser.parseFromString("<div xmlns:p='Edi' xmlns='http://www.w3.org/1999/xhtml'>" + html + "</div>", "text/xml");
            return domDoc;
        }
    }
    catch(e) {
        throw "Error in preparePrint, in searchSystem " + this.id +
            ": " + e;
    }
};

/* does a lookup in an LC */
function getApproxIdSearch(sSearch, pathTable, pathIndex, bSelect){
    try{
        var oTable = skTabManager.getSkTable(pathTable);
        sSearch = g_Simplifier.Simplify(sSearch.replace(/[ \.\*\?;!()\+,\[:\]<>^_`\-\[\]{}~]/g,""));
        var realId = 0;
        var selectedId = 0;
        var bExact = false;
    }catch(e){
        throw("getApproxIdSearch : " + e);
        return [0,0,false];
    }
    try{
        realId = oTable.LookupText(sSearch,
            Components.interfaces.skXIRecordSet.lmEXACTORAFTER);
        var rs = skTabManager.getFieldValue(pathTable, realId, "a_ids");
        var r = rs.GetRecord(0);
        realId = r.GetUNumFieldValue(skTabManager.getField(pathTable + "/a_ids.skn", "a_id"));
        selectedId = realId;

        if(!bSelect)
            if((realId-3)>0)
                selectedId = realId - 3;
        bExact = true;
    }catch(e){
        jsLog("Error in getApproxIdSearch:" + e);
    }
    return [realId, selectedId, bExact];
}

/**********************************************
 * Wrapper on skVlist
 **********************************************/
function VList(elem, selectCallback) {
    this.skVlist = elem;
    var me = this;

    this.focus = function() {
        this.skVlist.focus();
    }

    // Assigns or returns the callback which displays an item at a given index
    this.getContentCallback = function(callback) {
        if (callback == null)
            return this.skVlist.myFuncGetContent;
        this.skVlist.myFuncGetContent = callback;
    }
    // Assigns or returns the callback which displays the tooltips
    this.getTooltipCallback = function(callback) {
        if (callback == null)
            return this.skVlist.myFuncGetTooltipContent;
        this.skVlist.myFuncGetTooltipContent = callback;
    }
    // Assigns or returns the callback which is triggered when an item of the
    // list is selected
    this.selectItemCallback = function(callback) {
        if (callback == null)
            return this.skVlist.onselectItem;
        this.skVlist.onselectItem = callback;
    }
    // Assigns or returns the callback triggered when an item of the list is
    // double-clicked
    this.dblClickItemCallback = function(callback) {
        if (callback == null)
            return this.skVlist.ondblClickItem();
        this.skVlist.ondblClickItem = callback;
    }
    // Assigns or returns the callback triggered when the mouse is moved over
    // an item of the list
    this.mouseMoveItemCallback = function(callback) {
        if (callback == null)
            return this.skVlist.onMouseMoveItem;
        this.skVlist.onMouseMoveItem = callback;
    }
    // Assigns or returns the callback triggered when the mouse any key
    // is pressed
    this.keyPressVlistCallback = function(callback) {
        if (callback == null)
            return this.skVlist.onKeyPressVlist;
        this.skVlist.onKeyPressVlist = callback;
    }
    // Assigns or returns the last index of the list
    this.lastIndex = function(nbMax) {
        if (nbMax == null)
            return this.skVlist.maxElement;
        this.skVlist.maxElement = nbMax;
    }
    // Assigns or returns the selected index of the list
    this.selectedIndex = function(idx) {
        if (idx == null)
            return this.skVlist.selectedIndex;
        this.skVlist.selectedIndex = idx;
    }
    // Makes sure something is always selected
    this.stickySelection = function(bSticky) {
        if (bSticky == null)
            return this.skVlist.stickySelection; 
        this.skVlist.stickySelection = bSticky;
    }
    // Upon loading the list, automatically selects the first element unless
    // selectedIndex is positioned
    this.selectFirstItem = function(bSelect) {
        if (bSelect == null)
            return this.skVlist.selectfirstitem;
        this.skVlist.selectfirstitem = bSelect;
    }
    // Empties the list
    this.clearList = function() {
        this.skVlist.clearList();
    }
    this.selectedItem = function() {
        return this.skVlist.selectedItem;
    }

    this.selectedIndex = function() {
        return this.skVlist.selectedIndex;
    }
    // Scrolls the list to a given position
    this.placeInVlist = function(id, context, highlight, force) {
        this.skVlist.scrollId = null;
        if (highlight)
            this.skVlist.highlight = highlight;
        if (context)
            this.skVlist.scrollId = context;
        try{
            if(force){
                this.skVlist.clearList();
                this.skVlist.selectedIndex = id;
                this.skVlist.load();
            }
            this.skVlist.loadFromIndex(id);
            // hack to force the virtual list to update selectedIndex when it
            // has only one element, otherwise it just works the first time
            // (useful for the go to entry function in the double click popup)
            // *** Doesn't seem useful after all
            /*if (this.lastIndex() == 0)
                this.skVlist.selectedIndex = id;*/
        }catch (e){
            jsLog(e)
        }
    }

    this.selectItemCallback(function(item, obj){});
    this.mouseMoveItemCallback(function(item, obj){});
    this.dblClickItemCallback(function(item, obj){});
    this.keyPressVlistCallback(function(obj){});
    // populates the virtual list
    this.load = function() {
        this.skVlist.load();
    }
    // Returns the item at id index
    this.get = function(index) {
        return this.skVlist.myFuncGetContent(index, this.skVlist);
    }
    this.singleEntry = function() {
        this.skVlist.collapScroll = "true";
    }
    // Returns the item at id index
    this.getCurrent = function() {
        return this.get(this.skVlist.selectedIndex);
    }
    this.fs = function(fs) {
        this.skVlist.fs = fs;
    }
    this.hide = function() {
        this.skVlist.collapsed = true;
    }
    this.scrollSpeed = function(speed) {
        if (speed)
            this.skVlist.scrollSpeed = speed;
        else
            return speed;
    }
}
