/**
 * Universal resource manager. A resource is roughly equivalent to a filesystem,
 * and used in cross-references in the application:
 *
 *     <Ref resource="resource_name" topic="entry_id" bookmark="scroll_id"/>
 *
 * Works with the skCrossref binding, to get a universal way of opening a cross
 * referencefrom anywhere in the application. An skResourceManager is defined
 * in the application upon inclusion
 *
 * Usage:
 *     skResourceManager.dataPrefix = "<appname>.data" // directory holding 
 * the data
 *
 *    skResourceManager.setHandler("some_resource", function(xrefElement,
 *        resource, topic, bookmark) { openPopup(...) }
 *
 * It's also possible to set a "self handler" for internal cross references
 * (when the origin resource of the crossreference is the same as the target
 * resource), with skResourceManager.setSelfHandler().
 * There are helper methods to set the most common handlers:
 * setScrollHandler, setScrollSelfHandler - the vlist will scroll to the right
 * point
 *
 * setPopupHandler, setPopupSelfHandler - a popup will open with the xref
 *
 * Finally, you can use skResourceManager.prefix(path) to add the dataPrefix to
 * a path if needed
 */
function skResourceManager() {
    this.init();
}

/**
 * Constructor
 */
skResourceManager.prototype.init = function () {
    this.dataPrefix = null;
    this.handlers = {}; // callbacks to display resources
    this.selfHandlers = {}; // callbacks to display internal xref
    this.groupIds = {}; // matches a resource with a group id (cf grouping.js)
    this.paths = {}; // aliases to resource pathes
        // handler)
    this.defaultPopupSettings = new skPopupSettings();
    this.popups = {} // popup setting for a given resource
    this.defaultSelfHandler = this.scrollHandler;
    this.defaultHandler = this.popupHandler;
}

/**
 * Adds a handler to display a given resource (or array of resources)
 */
skResourceManager.prototype.setHandler = function(resource, callback) {
    if (resource instanceof Array)
        for (i in resource)
            this.handlers[resource[i].toLowerCase()] = callback;
    else
        this.handlers[resource.toLowerCase()] = callback;
}

/**
 * Adds a handler to display a given resource (or array of resources) - when the
 * resource of the document requesting show() is the same as the resource to
 * display (in most cases, you want to fall back to the default handler, which
 * will attempt to scroll in the virtual list)
 */
skResourceManager.prototype.setSelfHandler = function(resource, callback) {
    if (resource instanceof Array)
       for (i in resource)
            this.selfHandlers[resource[i].toLowerCase()] = callback;
    else
        this.selfHandlers[resource.toLowerCase()] = callback;
}

/**
 * Convenience method to set a scroll handler
 */
skResourceManager.prototype.setScrollHandler = function(resource, groupId) {
    this.groupIds[resource.toLowerCase()] = groupId;
    this.setHandler(resource, this.scrollHandler);
}
/**
 * Convenience method to set a scroll self handler
 * No corresponding method for setting a scroll handler because this wouldn't
 * make sense
 */
skResourceManager.prototype.setScrollSelfHandler = function(resource) {
    this.setSelfHandler(resource, this.scrollHandler);
}
/**
 * Convenience method to set a popup handler
 * @param resource the name of resource
 * @param settings a hash of options for skPopupSettings, or an skPopupSettings
 * object, or the url of popup
 */
skResourceManager.prototype.setPopupHandler = function(resource, settings) {
    var popupSettings = this.makePopupSettings(settings);
    this.popups[resource.toLowerCase()] = popupSettings;
    this.setHandler(resource, this.popupHandler);
}

/**
 * Convenience method to set a popup self handler
 * @param resource the name of resource
 * @param settings a hash of options for skPopupSettings, or an skPopupSettings
 * object
 */
skResourceManager.prototype.setPopupSelfHandler = function(resource, settings) {
    var popupSettings = this.makePopupSettings(settings);
    this.popups[resource.toLowerCase()] = popupSettings;
    this.setSelfHandler(resource, this.popupHandler, popupSettings);
}

skResourceManager.prototype.makePopupSettings = function(settings) {
    if (!settings)
        return this.defaultPopupSettings;
    else if (settings instanceof skPopupSettings)
        return settings;
    else if (settings instanceof Object)
        return new skPopupSettings(settings);
    else if (typeof settings == "string" || settings instanceof String)
        return new skPopupSettings({url: settings})
    else
        throw "makePopupSettings: Don't know what to do with these settings: " + settings;
}

/**
 * Tries to display a resource - called from the XBL in skCrossref
 */
skResourceManager.prototype.showResource = function(xRefElement) {
    try {
        // tries to determine to which resource the source belongs
        var srcResource;
        try {
            srcResource = this.getContainerResource(xRefElement)
        }
        catch(e) {
            srcResource = null;
            if (!srcResource)
                jsLog("WARNING: Can't determine to which resource this crossref element belongs: " + e);
        }
        var targetResource = xRefElement.resource ||
            xRefElement.getAttribute("resource");
        var topic = xRefElement.topic ||  xRefElement.getAttribute("topic");
        var bookmark = xRefElement.bookmark ||
            xRefElement.getAttribute("bookmark");
        this.showExplicitResource(xRefElement, srcResource, targetResource, topic, bookmark);
    }
    catch(e) {
        jsLog("Error in skResourceManager.showResource: " + e);
        throw e;
    }
}

/**
 * Sends explicitly the source and target resources, as well as the topic and
 * bookmark values, instead of deducing them
 */
skResourceManager.prototype.showExplicitResource = function(xRefElement, srcResource, targetResource, topic, bookmark) {
    try {
        if (srcResource == null && targetResource == null)
            throw "Can determine neither source nor target resource for this crossref";
        // are we trying to display a resource in the same context
        var isSelfResource = srcResource != null && (targetResource == null ||
                srcResource == targetResource);
        var callback;
        if (isSelfResource)
            callback = this.selfHandlers[srcResource.toLowerCase()] ||
                this.defaultSelfHandler;
        else
            callback = this.handlers[targetResource.toLowerCase()] ||
                this.defaultHandler;
        if (!callback)
            throw "Can't find callback for this crossref element";
        // also pass the resource to the callback, in case it's generic
        targetResource = targetResource || srcResource;
        try {
            callback(xRefElement, targetResource, topic, bookmark);
        }
        catch(e) {
            throw "Error triggering callback: " + e;
        }
    }
    catch(e) {
        jsLog("Error in skResourceManager.showExplicitResource: " + e);
        throw e;
    }

}

/**
 * Tries to find to which resource this physical path is mapped
 */
skResourceManager.prototype.resolvePath = function (path) {
    // reverse lookup in the paths table
    for (resource in this.paths) {
        if (path == this.paths[resource])
            return resource;
    }
    // Didn't find anything, try path munging
    return this.unprefix(path).
        replace(/\/[^\/]+?\.(?:cff|cfi)$/, ""). // trash the config.cff, etc
        replace(/.skn$/, ""). // trash final .skn
        replace(/\.skn\//g, "__"); // .skn/ -> __
}

/**
 * Tries to resolve the physical path of a resource
 */
skResourceManager.prototype.resolveResource = function(resource) {
    var lcResource = resource.toLowerCase();
    if (this.paths[lcResource])
        return this.prefix(this.paths[lcResource]);
    else
        return this.prefix(lcResource.replace(/__/g, ".skn/")
                + ".skn");
}

/**
 * Adds the data prefix (the name of the data directory) to the path
 */
skResourceManager.prototype.prefix = function(path) {
    return this.dataPrefix ? this.dataPrefix + "/" + path : path;
}

/**
 * Removes the data prefix (the name of the data directory) from the path
 */
skResourceManager.prototype.unprefix = function(path) {
    return this.dataPrefix ? path.replace(new RegExp("^" + this.dataPrefix + "/"), "") : path;
}

/**
 * Adds a path to a resource (per default, we try dataPrefix + "/" + resource
 * name in lowercase + ".skn" when trying to resolve it
 */
skResourceManager.prototype.addPath = function(resource, path) {
    this.paths[resource.toLowerCase()] = path;
}

/**
 * Returns the container of a given element, or null
 */
skResourceManager.prototype.getContainer = function(element) {
    if (element.ownerDocument && element.ownerDocument.defaultView.vList)
        return element.ownerDocument.defaultView.vList;
    else
        return null;
}

/**
 * Returns the resource of the container (virtual list) of a given element,
 * or null
 */
skResourceManager.prototype.getContainerResource = function(element) {
    try {
        var container = this.getContainer(element);
        if (!container)
            throw "No container found";
        var fs = container.fs || container.getAttribute("fs");
        if (!fs)
            throw "Can't determine filesystem of container";
        return this.resolvePath(fs);
    }
    catch(e) {
        jsLog("Error in skResourceManager.getContainerResource: " + e);
        throw e;
    }
}

/**
 * Tries to activate the correct group (if a group has been associated to the resource by setScrollHandler).
 * Displays the target entry in the virtual list of the searchSystem associated with the groupId.
 * In the case of an internal xref (or if no group id has been specified), displays the target
 * entry in the same vlist as the source.
 */
skResourceManager.prototype.scrollHandler = function (element, resource, topic,
    bookmark) {
    try {
        var me = skResourceManager;
        var idmId = me.getTargetId(resource, topic);
        if (!idmId) // oups
            throw "No IDM id matching " + topic;
        // try to activate the associated group, if any
        var groupId = null;
        if (resource)
            groupId = me.groupIds[resource.toLowerCase()];
        if (!groupId) {
            // internal xref? try to scroll in the same container
            var container = me.getContainer(element);
            if (!container)
                throw "Can't find container for this element";
            container.scrollId = bookmark;
            container.loadFromIndex(idmId);
        }
        else {
            var group = skGroups.group(groupId);
            if (!group)
                throw "Couldn't find group of id '" + groupId + "'";
            group.enable(groupId);
            // scroll the main az
            group.searchSystem.azList.placeInVlist(idmId, bookmark);
        }
    }
    catch (e) {
        jsLog("Error in skResourceManager.defaultSelfHandler: " + e);
        throw e;
    }
}

/**
 * Returns the IDM id matching a topic, or null. Throws an error if the
 * resource can't be resolved
 */
skResourceManager.prototype.getTargetId = function(resource, topic) {
    var me = skResourceManager;
    var fs = me.resolveResource(resource);
    if (!fs)
        throw "Can't resolve resource " + resource;
    // Topic is an ID from the publisher, lookup the internal id
    var idmId = skTabManager.getIdMapping(fs,
        topic);
    return idmId;
}

/**
 * Displays the crossref in a popup
 */
skResourceManager.prototype.popupHandler = function (element, resource, topic,
    bookmark) {
    try {
        var me = skResourceManager;
        var popupSettings = me.popups[resource.toLowerCase()] ||
            me.defaultPopupSettings; 
        if (!skPopup)
            throw "Can't find skPopup";
        var fs = me.resolveResource(resource);
        if (!fs)
            throw "Can't resolve resource " + resource;

        var targetId = me.getTargetId(resource, topic);
        if (!targetId)
            throw "No IDM id matching " + topic;

        skPopup.closePopupById(popupSettings.popupId);
        if (popupSettings.closeAllPopups)
            skPopup.closePopupById(popupSettings.popupId);

        if (popupSettings.closeAllPopups)
            skPopup.closeAllPopups();
        var newPopup = skPopup.initNewPopup(popupSettings.popupId,
                "POPUP_TYPE_GENERIC");

        // pass everything relevant to the popup
        newPopup.extraArgs = {
            element: element,
            resource: resource,
            topic: topic,
            bookmark: bookmark,
            idmId: targetId
        }
        newPopup.url = popupSettings.url;
        newPopup.width  = popupSettings.width;
        newPopup.height = popupSettings.height;
        var mainWindow = window;
        var newWindow = skPopup.openPopup();
        newWindow.addEventListener("load", function() {
            var xrefParent = newWindow.document.getElementById("xrefParent");
            if (!xrefParent)
                throw "Couldn't find the element of id 'xrefParent'";

            // if the target is a vlist, change its initcallback
            if (xrefParent.nodeName == "skVirtualList") {
                var vlist = new VList(xrefParent);
                vlist.lastIndex(0);
                vlist.getContentCallback(function (piIndex, obj) {
                    var entry = skTabManager.getFile(fs + "/config.cff",
                        piIndex).GetData().GetWStringData();
                    entry = entry.replace(/<\?xml[^>]*>/, "");
                    return entry;
                });
                // tell the vlist what fs it's using, otherwise the xref won't
                // work inside it

                // getTargetId returns a full path, including dataPrefix
                // scrap it
                vlist.fs(me.unprefix(fs));
                vlist.placeInVlist(targetId, bookmark);
            }
            else {
                // Otherwise just append to xrefParent
                var entry = skTabManager.getFile(fs + "/config.cff",
                        targetId).GetData().GetWStringData();
                var parser = new DOMParser();
                var parsedElement = parser.parseFromString(entry,
                        "application/xml");
                xrefParent.appendChild(parsedElement.documentElement);
            }
            // Insert the title (can be string or callback)
            if (typeof popupSettings.title == "string" || 
                    popupSettings.title instanceof String)
                titleToInsert = popupSettings.title;
            else if (typeof popupSettings.title == "function")
                titleToInsert = popupSettings.title(element, resource, topic,
                        bookmark);
            var doc = newWindow.document;
            // XUL window
            if (doc.documentElement.nodeName == "window")
                doc.documentElement.setAttribute("title", titleToInsert);
            // HTML window
            else if (doc.documentElement.nodeName == "html") {
                var head = null;
                var heads = doc.getElementByTagName("head");
                if (heads.length > 0)
                    head = heads[0];
                else {
                    // couldn't find the <head> tag, create it
                    head = doc.createElementNS(doc.documentElement.namespaceURI, "head");
                    doc.documentElement.insertBefore(head, doc.documentElement.firstChild);
                }
                // find or create the title element and insert the content 
                var title = title.getElementsByTagName("title")[0];
                if (!title) {
                    title = doc.createElementNS(doc.documentElement.namespaceURI, "title");
                    head.appendChild(title);
                }
                title.nodeValue = titleToInsert;
            }
        }, false);
    }
    catch (e) {
        jsLog("Error in skResourceManager.popupHandler: " + e);
        throw e;
    }
}

// Creates the resource manager
var skResourceManager = new skResourceManager();

/**
 * Holds the settings for the popupHandler popups
 * Should have sensible defaults
 * See below for valid options
 */
function skPopupSettings(opts) {
    // create empty opts if needed, this way we can just use || instead of the
    // ternary operator
    opts = opts || {};

    this.popupId = opts.popupId || "XREF"; // id of the xref popup (used when by the popup
    this.url = opts.url || "chrome://plugs/content/skCrossref/popup/skCrossref.xul";
    this.width = opts.width || 300;
    this.height = opts.height || 500;
    this.closeAllPopups = opts.closeAllPopups || false; // close all other popups when opening this one?
    this.closePopupFromMain = opts.closePopupFromMain != null ?
        opts.closePopupFromMain :
        true; // close this popup when clicking on the main window?
    // TODO implement closePopupFromMain
    this.title = opts.title || "Definition"; // static title or callback (takes 'resource' and 'idmid' as parameters)
}
