/**
 * Creates an topic dictionary from a group of widgets
 * The Oxford topic dictionary is made of one indexes, a search box, a menu list,
 * a checkbox, a result list and an AZ list
 * For instance:
 *
      <!-- search box -->
      <textbox  id="topicSearch" group="topicSearch" autoscroll="true"/>
      <!-- triggers the search -->
      <button group="topicSearch"/>
      <!-- menulist -->
      <menulist id="topicFilter" group="topicSearch" fs="topic.skn/topic_entries.skn"/>
      <!-- checkbox -->
      <checkbox id="topicSwitchDisplay" group="topicSearch" fullcss="chrome://doas/skin/topic.css" simplecss="chrome://doas/skin/simple.css"/>

      <!-- alpha index -->
      <skVirtualList id="indexTopic"
        src="chrome://doas/content/iframe/azMD.xml" flex="1"
        group="topicSearch" type="index"
        fs="topics.skn/alpha_index.skn"/>

      <!-- result list -->
      <skVirtualList id="searchResultTopic"
        src="chrome://doas/content/iframe/resultMD.xml" flex="1"
        group="topicSearch" type="results"
        fs="topic.skn/arl.skn"/>

      <!-- az list -->
      <skVirtualList id="contentTopic"
        src="chrome://doas/content/iframe/topic.xml" flex="1"
        group="topicSearch" type="az"
        fs="topics.skn/fs.skn"/>
 */
 
function TopicSearchSystem(id, opts) {
    // JS inheritance
    this.base = SearchSystem;
    this.base(id, opts);
    var me = this;
    this.filterEntries = {}; //hash that contains the list of topic entries already loaded
    this.initMenulist();
    this.initCheckbox();

    this.indexList.selectItemCallback(function(item, obj) {
        try{
            var searchString;
            // the index might have a special ARL form
            try {
                searchString = skTabManager.getFieldValue(me.fsIdx, item.index, "ARL");
            } catch(e) {
                // otherwise, use the lookup form
                searchString = skTabManager.getFieldValue(me.fsIdx, item.index, "SRCH");
            }
            var guidToSelect = skTabManager.getFieldValue(me.fsIdx, item.index, "ID");
            me.entryToSelect = guidToSelect;
            // scroll in the main az list if needed
            if (!me.searchFromIndex && me.azList)
                me.azList.placeInVlist(guidToSelect, null, null, true);
            if (me.searchFromIndex)
                me.searchAndDisplay(me.removeWildcards(searchString), null, guidToSelect, false);
        } catch (err) {
            jsLog(err)
        }
    });

    this.resultList.selectItemCallback(function(item, obj){
        try {
            var topicName = skTabManager.getFieldValue(me.fsResults, me.searchResults[item.index], "topic");
            var topicIdx = me.getTopicIndex(topicName);
            me.filterAzList(topicIdx);
        } catch (err) { jsLog(err) }
    });

    this.azList.getContentCallback(function (index, obj) {
        try{
            if (me.filtered && me.filterEntries["filter" + me.filterIndex])
               index = Number(me.filterEntries["filter" + me.filterIndex][index]);

            var entry = skTabManager.getFile(me.fsAz + "/config.cff", index).GetData().GetWStringData();
            entry = entry.replace(/^\s*<\?xml.+\??>/, '');
            if(this.note)
                entry = this.note(entry);
            return String(entry);
        } catch (err) {
            jsLog("GetEntryContent failed: " + err + " ON piIndex : " + piIndex);
        }
    });

    this.azList.getTooltipCallback(function (index, obj){
        try{
            if (me.filtered && me.filterEntries["filter" + me.filterIndex])
               index = Number(me.filterEntries["filter" + me.filterIndex][index]);

            var entry = skTabManager.getFieldValue(me.fsAz + "/files.skn", index, "TEXTTITLE");
            entry = entry.replace(/&amp;/g,"&");
            if(this.note)
                entry = this.note(entry);
            return String(entry);
        } catch (err) {jsLog(err) }
    });

}

TopicSearchSystem.prototype = new SearchSystem;

TopicSearchSystem.prototype.assignWidget = function (widget) {
    try {
        // sacrifices a chicken by a moonless night to the Ancient Ones,
        // to simulate 'super'
        var type = $(widget).attr("type");
        var nodeName = $(widget).nodeName();
        if (!SearchSystem.prototype.assignWidget.call(this, widget)) {
            if (nodeName == "menulist") {
                this.menulist = widget;
            } else
                return false;
            widget.searchSystem = this;
            return true;
        }
        if (type == "switchcheckbox") {
            this.switchCheckbox = widget;
            this.checkboxes = []; //empty the checkboxes array
        }
        return true;
    }
    catch (e) {
        throw "Error in TopicSearchSystem.assignWidget, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}

TopicSearchSystem.prototype.initMenulist = function(widget) {
    try {
        if (!this.menulist)
            return;

        var me = this;
        this.topics = new Array();
        var i;
        
        //sort topics
        this.fsTopic = this.prefix($(this.menulist).attr("fs")); // "topic.skn/topic_entries.skn"
        for (i=0; i<skTabManager.getCount(this.fsTopic); i++)
            this.topics.push( { label: skTabManager.getFieldValue(this.fsTopic, i, "id"), idx: i } );
        this.topics.sort(function(a, b) { return (a.label < b.label ? -1 : 1) });
        
        this.menulist.addEventListener("command",
                                       function() {
                                           me.filterAzList(this.selectedItem.getAttribute("idx"));
                                       }, true);

        //fill the menulist
        var menupopup = document.createElement("menupopup");
        var i;
        for (i=0; i<this.topics.length; i++) {
            var filterId = skTabManager.getFieldValue(this.fsTopic, i, "id");
            var menuitem = document.createElement("menuitem");
            menuitem.setAttribute("idx", this.topics[i].idx);
            menuitem.setAttribute("label", this.topics[i].label);
            menupopup.appendChild(menuitem);
        }
        this.menulist.appendChild(menupopup);

        //select the first topic
        this.filterAzList(this.topics[0].idx);
    } catch(e) {
        throw "Error in initMenulist, in topicSearchSystem " + this.id + 
            ":" + e;
    }
};

TopicSearchSystem.prototype.initCheckbox = function(widget) {
    try {
        if (!this.switchCheckbox)
            return;

        var me = this;
        this.switchCheckbox.addEventListener("command",
                                             function() {
                                                me.switchDisplay(!this.checked);
                                             } , true);

        this.fullCss = $(this.switchCheckbox).attr("fullcss");
        this.simpleCss = $(this.switchCheckbox).attr("simplecss");
    } catch(e) {
        throw "Error in initCheckbox, in topicSearchSystem " + this.id + 
            ":" + e;
    }
}

//switch the display mode of the ZA list (full entries or  headwords only)
TopicSearchSystem.prototype.switchDisplay = function(showFullEntry) {
    try {
        if (!this.switchCheckbox)
            return;
        
        var css = (showFullEntry ? this.fullCss : this.simpleCss);
        this.azList.skVlist.addCssArray([css]);
    } catch(e) {
        throw "Error in switchDisplay, in topicSearchSystem " + this.id + ":" + e;
    }
};

//return the index (its position in topic_entries) of the given topic by its name (topic id)
TopicSearchSystem.prototype.getTopicIndex = function(name) {
    try {
        for (i=0; i<this.topics.length; i++) {
            if (this.topics[i].label == name)
                return this.topics[i].idx;
        }
        return -1;
    } catch(e) {
        throw "Error in getTopicIndex, in topicSearchSystem " + this.id + ":" + e;
    }
};

//return the index (its alphabetical position from 0 to n) of the given topic by its "non-alphabetical index" (its position in topic_entries)
TopicSearchSystem.prototype.getTopicAlphaIndex = function(idx) {
    try {
        for (i=0; i<this.topics.length; i++) {
            if (this.topics[i].idx == idx)
                return i;
        }
        return -1;
    } catch(e) {
        throw "Error in getTopicAlphaIndex, in topicSearchSystem " + this.id + ":" + e;
    }
};

TopicSearchSystem.prototype.filterAzList = function(filterIndex) {
    try {
        if(!this.az)
            return;

        this.filterIndex = filterIndex;
        var me = this;

        //select the topic in the menulist
        this.menulist.selectedIndex = this.getTopicAlphaIndex(filterIndex);

        //get entries that belongs to the selected topic
        //and use a cache (filterEntries) to retrieve them later
        if (!this.filterEntries["filter" + filterIndex]) {
            var i;
            var ids = [];
            var entries = skTabManager.getFieldValue(this.fsTopic, filterIndex, "entries");
            for (i=0; i<entries.GetCount(); i++) {
                var entry_id = skTabManager.getFieldValue(this.fsTopic + "/entries.skn", null, "entry", entries.GetRecord(i));
                ids.push(entry_id);
            }
            this.filterEntries["filter" + filterIndex] = ids;
            //jsLog(">> " + this.filterEntries["filter" + filterIndex]);
        } else {
            //jsLog(">> [CACHED]" + this.filterEntries["filter" + filterIndex]);
        }

        //fill azList
        this.azList.lastIndex(this.filterEntries["filter" + filterIndex].length-1);
        this.azList.clearList();

        //calculate the index of the AZ entry to scroll to.
        var azIndex = 0
        if (this.entryToSelect != null) {
            var topicEntries = me.filterEntries["filter" + filterIndex];
            var i;
            for (i=0; i<topicEntries.length; i++) {
                if (topicEntries[i] == this.entryToSelect) {
                    azIndex = i;
                    break;
                }
            }
            this.entryToSelect = null;
        }
        this.azList.skVlist.selectedIndex = azIndex;
        this.filtered = true;
        me.azList.load();
        me.placeInAz(azIndex);
    } catch (err) {
       jsLog("ERROR function filterAzList:\n" + err);
    }
};

// Make sure that 'enter' can trigger the search and the scroll of the index
TopicSearchSystem.prototype.initInput = function() {
    try {
        var me = this;
        $(this.input).keyup(function(e) {
            if (e.keyCode == 13){
                if (me.indexScrollId)
                    me.entryToSelect = skTabManager.getFieldValue(me.fsIdx, me.indexScrollId, "ID");
                me.searchAndDisplay(null, null, me.entryToSelect, false);
                if (me.azList)
                    me.azList.skVlist.focus();
            }else{
                if($(me.input).attr("autoscroll")=="true"){
                    me.scroll();
                }
            }
        });
    }
    catch(e) {
        throw "Error in initInput, in topicSearchSystem " + this.id + 
            ":" + e;
    }
};

/** Makes the button trigger the search */
TopicSearchSystem.prototype.initButton = function() {
    if (!this.button || $(this.button).hasAttr("onclick"))
        return;
    try {
        // avoid weird problems with 'this' in Javascript
        var me = this;
        $(this.button).click(function() {
            var entryToSelect = null;
            if (me.indexScrollId)
                me.entryToSelect = skTabManager.getFieldValue(me.fsIdx, me.indexScrollId, "ID");
            me.searchAndDisplay(null, null, me.entryToSelect, false);
        });
    }
    catch(e) {
        throw "Error in initButton, in topicSearchSystem " + this.id + 
            ": " + e;
    }
};
